class ConfigError(Exception):
    pass


class GroupSecureKeyError(Exception):
    pass


class UploadFileError(Exception):
    pass


class UploadFileSizeError(Exception):
    pass


class SystemError(Exception):
    pass
